@@ -39,9 +39,17 @@
 
 #include <memory>
 
+// Forward decl
 class OGRGeometry;
+
+// Get GDAL's forward decls if available
+// otherwise make our own
+#if __has_include(<gdal_fwd.h>)
+#include <gdal_fwd.h>
+#else
 using OGRGeometryH = void *;
 using OGRSpatialReferenceH = void *;
+#endif
 
 namespace pdal
 {
