@@ -35,10 +35,12 @@
 #pragma once
 
 #include <pdal/pdal_features.hpp>
+
+#include "HeaderVal.hpp"
+
 #include <pdal/FlexWriter.hpp>
 #include <pdal/Streamable.hpp>
 
-#include "HeaderVal.hpp"
 #include "private/las/Vlr.hpp"
 
 namespace pdal
@@ -105,8 +107,7 @@ private:
     virtual void initialize();
     virtual void prepared(PointTableRef table);
     virtual void readyTable(PointTableRef table);
-    virtual void readyFile(const std::string& filename,
-        const SpatialReference& srs);
+    virtual void readyFile(const std::string& filename, const SpatialReference& srs);
     virtual bool srsOverridden() const;
     void prerunFile(const PointViewSet& pvSet);
     virtual void writeView(const PointViewPtr view);
@@ -122,6 +123,7 @@ private:
         const MetadataNode& base);
     void handleHeaderForwards(MetadataNode& forward);
     void fillHeader();
+    void validateHeader();
     bool fillPointBuf(PointRef& point, LeInserter& ostream);
     point_count_t fillWriteBuf(const PointView& view, PointId startId,
         std::vector<char>& buf);
