@@ -39,7 +39,16 @@
 #include <pdal/util/Bounds.hpp>
 
 class OGRGeometry;
-typedef void *OGRGeometryH;
+
+// Get GDAL's forward decls if available
+// otherwise make our own
+#if __has_include(<gdal_fwd.h>)
+#include <gdal_fwd.h>
+#else
+using OGRGeometryH = void *;
+using OGRSpatialReferenceH = void *;
+#endif
+
 
 #include <vector>
 
@@ -68,6 +77,7 @@ PDAL_DLL std::vector<Polygon> getPolygons(const NL::json& ogr);
 // New signatures to support extraction of SRS from the end of geometry
 // specifications.
 OGRGeometry *createFromWkt(const std::string& s, std::string& srs);
+OGRGeometry *createFromWkb(const std::string& s, std::string& srs);
 OGRGeometry *createFromGeoJson(const std::string& s, std::string& srs);
 
 inline OGRGeometry *fromHandle(OGRGeometryH geom)
