@@ -90,7 +90,7 @@ namespace Utils
             else
                 m_code = -1;
         }
-        StatusWithReason(int code);  // Not defined
+        StatusWithReason(int code) = delete;
         StatusWithReason(int code, const std::string& what) :
             m_code(code), m_what(what)
         {}
@@ -216,6 +216,8 @@ namespace Utils
     */
     inline bool startsWith(const std::string& s, const std::string& prefix)
     {
+        if (prefix.empty())
+            return true;
         if (prefix.size() > s.size())
             return false;
         return (strncmp(prefix.data(), s.data(), prefix.size()) == 0);
@@ -230,6 +232,8 @@ namespace Utils
     */
     inline bool endsWith(const std::string& s, const std::string& postfix)
     {
+        if (postfix.empty())
+            return true;
         if (postfix.size() > s.size())
             return false;
         return (strcmp(postfix.data(),
@@ -463,17 +467,6 @@ namespace Utils
     */
     PDAL_DLL double normalizeLongitude(double longitude);
 
-    /**
-      Convert an input buffer to a hexadecimal string representation similar
-      to the output of the UNIX command 'od'.  This is mostly used as an
-      occasional debugging aid.
-
-      \param buf  Point to buffer to dump.
-      \param count  Size of buffer.
-      \return  Buffer converted to hex string.
-    */
-    PDAL_DLL std::string hexDump(const char *buf, size_t count);
-
     /**
       Count the number of characters in a string that meet a predicate.
 
